#include "Vertor_PCA.h"

Vertor_PCA::~Vertor_PCA() {
    if (this->clientI2C)
        delete this->clientI2C;
}

void Vertor_PCA::init() {
    this->clientI2C = new I2C();
    I2CTransport::setClient(this->clientI2C);
    I2CTransport::begin();
    PCA9685::begin();

    /* Zero servo */
    for (uint8_t i = 0; i < 8; i++) {
        this->writeServo(i, 0);
    }

    /* Zero motors */
    for (uint8_t i = 0; i < 4; i++) {
        this->writeDCMotor(i, 0, 0);
    }
}

void Vertor_PCA::setRangeUs(uint16_t min, uint16_t max) {
    this->min_us = min;
    this->max_us = max;
}

void Vertor_PCA::writeServo(uint8_t servoID, uint16_t angle) {
    if (servoID < 8 || servoID > 15)
        return;

    uint16_t length = this->min_us + (this->max_us - this->min_us) / 180 * angle;
    /* 20 ms / 4096 = 4.883 us */
    length /= 4.883;
    PCA9685::setPWM(servoID, 0, length);
}

void Vertor_PCA::writeDCMotor(uint8_t motorID, uint8_t pwm, uint8_t reverse) {
    /* Motor ID must be 0..3 */
    if (motorID > 3)
        return;

    /* Define pins */
    uint8_t ch1_pin = motorID*2;
    uint8_t ch2_pin = ch1_pin + 1;

    if (reverse) {
        /* Backward */
        PCA9685::setPWM(ch1_pin, 0);
        PCA9685::setPWM(ch2_pin, pwm);
    } else {
        /* Forward */
        PCA9685::setPWM(ch1_pin, pwm);
        PCA9685::setPWM(ch2_pin, 0);
    }
}

