#ifndef DCMOTOR_H
#define DCMOTOR_H

#include <Board.h>
#include "Encoders/OpticalTickEncoder.h"
#include "Encoders/HallTickEncoder.h"

enum MotorState {
    STATE_STOPPED,
    STATE_FORWARD,
    STATE_BACKWARD
};

enum MotorScheduledTask {
    MOTOR_TASK_IDLE,
    MOTOR_TASK_OFF
};

class DCMotor {
    public:
        DCMotor(uint8_t _id, uint8_t _gear, bool _reverse = false, uint8_t _inertFactor = 0);
        void forward(uint8_t speed);
        void forward(uint8_t speed, float rotations);
        void backward(uint8_t speed);
        void backward(uint8_t speed, float rotations);
        void stop(bool hard = true);
        void abort();
        bool isStopped();
        bool isForward();
        bool isBackward();
        
        bool isBusy();
        void externalTick(uint16_t periodMs);
        void attachEncoder(BaseTickEncoder* _BaseTickEncoder);
        void attachEncoder(OpticalTickEncoder* _OpticalTickEncoder);
        void attachEncoder(HallTickEncoder* _HallTickEncoder);
        void setForwardSpeed(uint8_t speed);
        void setBackwardSpeed(uint8_t speed);
        uint8_t getGear();

    private:
        // Encoder on wheel
        BaseTickEncoder* encoder = nullptr;

        volatile MotorState state = STATE_STOPPED;
        volatile MotorScheduledTask task = MOTOR_TASK_IDLE;

        uint8_t motorID = 0;
        uint8_t gear = 1;

        bool reverse = false;
        uint8_t inertFactor = 0;

        // Current motor state
        uint8_t forwardSpeed = 0;
        uint8_t backwardSpeed = 0;

        uint32_t msCounter = 0;
        uint32_t msLimit = 0;

        bool encoderEnabled = false;

        void off();
        bool isEncoderLimitExceeded();
        uint8_t getHardStopTime();
        uint8_t normalizeSpeed(uint8_t speed);
        void checkEncoderLimit();
        uint8_t getSpeed();
        void delayAndOff(uint8_t time_ms);
        void writeSpeed(uint8_t speed, bool _reverse);
};

#endif /* DCMOTOR_H */
