#include "BaseTickEncoder.h"

BaseTickEncoder::BaseTickEncoder(uint8_t _pulsesPerLoop) {
    this->pulsesPerLoop = _pulsesPerLoop;
    Tweak::attachUsMemberCallback(this, TICK_US);
}

void BaseTickEncoder::tweakHandler() {
    this->update();
}

void BaseTickEncoder::enable() {
    this->isEnabled = true;
}

void BaseTickEncoder::disable() {
    this->isEnabled = false;
}

void BaseTickEncoder::reset() {
    this->pulsesCounter = 0;
    this->pulsesTravelled = 0;
    this->pulsesLimit = 0;
    this->_isLimited = false;
}

int32_t BaseTickEncoder::getPulses() {
    return this->pulsesCounter;
}

uint32_t BaseTickEncoder::getDistanceTravelled(uint16_t wheelRadius) {
    uint16_t wheelLength = TWO_PI * wheelRadius;
    return wheelLength * this->pulsesTravelled / this->pulsesPerLoop;
}

uint32_t BaseTickEncoder::getDistanceTravelled(uint16_t wheelRadius, uint8_t gear) {
    uint16_t wheelLength = TWO_PI * wheelRadius;
    return wheelLength * this->pulsesTravelled / this->pulsesPerLoop / gear;
}

void BaseTickEncoder::resetDistanceTravelled() {
    this->pulsesTravelled = 0;
}

void BaseTickEncoder::setLimit(float rotations, bool isFwd) {
    this->reset();
    this->pulsesLimit = rotations * this->pulsesPerLoop;
    this->_isLimited = true;
    this->isForward = isFwd;
    this->enable();
}

bool BaseTickEncoder::isLimited() {
    return this->_isLimited;
}

bool BaseTickEncoder::isLimitExceeded() {
    // return this->isLimited() && abs(this->pulsesCounter) >= this->pulsesLimit;
    return this->isLimited() && this->pulsesLimit == 0;
}

uint16_t BaseTickEncoder::getRPM() {
    uint32_t pulse_us = TICK_US * (uint32_t)this->ticksPerPulse * (uint32_t)this->pulsesPerLoop; // us in 1 rotation
    /* 60s / period = 60000000 / pulse_us */
    return 60000000 / pulse_us;
}

uint16_t BaseTickEncoder::getAngleSpeed(uint8_t gear) {
    uint16_t rpm = this->getRPM() / gear;
    /* deg/sec = rpm * 360deg / 60s = rpm * 6 */
    return rpm * 6;
}