#ifndef BASE_TICK_ENCODER_H
#define BASE_TICK_ENCODER_H

#include <Tweak.h>

#define TICK_US     1000

class BaseTickEncoder : public TweakCallbackOwnerClass {
    public:
        BaseTickEncoder(uint8_t _pulsesPerLoop);
        void tweakHandler();
        void enable();
        void disable();
        void reset();
        int32_t getPulses();
        uint32_t getDistanceTravelled(uint16_t wheelRadius);
        uint32_t getDistanceTravelled(uint16_t wheelRadius, uint8_t gear);
        void resetDistanceTravelled();
        void setLimit(float rotations, bool isFwd);
        bool isLimited();
        bool isLimitExceeded();
        uint16_t getAngleSpeed(uint8_t gear);

        uint8_t name;

    protected:
        uint16_t getRPM();

        virtual void update() = 0;

        /* Feature of encoder type */
        uint8_t pulsesPerLoop = 1;
        /* To calculate period and freq */
        uint16_t ticksPerPulse = 0;
        uint16_t ticksPerPulseCounter = 0;
        /* To handle limitation */
        uint32_t pulsesLimit = 0;
        volatile bool _isLimited = false;
        /* To count distance */
        uint32_t pulsesTravelled = 0;
        /* For PID regulation */
        int32_t pulsesCounter = 0;

        bool isEnabled = false;
        bool isForward = true;
};

#endif /* BASE_TICK_ENCODER_H */