#ifndef _GEEGROW_TRUCK_MOTION_MATH_H
#define _GEEGROW_TRUCK_MOTION_MATH_H
#include <Arduino.h>

class MotionMath {
	public:

		static const uint16_t F_100_TWO_PI = 628; //3.14 * 2 * 100
		static const uint16_t F_100_DEG_360 = 36000; //360 deg * 100

		/**
		 * @param uint16_t distance - cm (6553 cm max = 65m)
		 * @param uint8_t wheelRadius - mm
		 */
		static float distanceToWheelRotations(uint16_t distance, uint8_t wheelRadius) {
			/*
			 * There are no faster variant for this formula
			 */
			return (float)( distance * 10)/(TWO_PI * wheelRadius);
		}

		/**
		 * Calculate how much wheel rotations need for wheel turn path
		 * @param uint8_t turnRadius - cm
		 * @param uint16_t angle - deg
		 * @param uint8_t wheelRadius - mm
		 */
		static float turningAngleToWheelRotations(uint8_t turnRadius, uint16_t angle, uint8_t wheelRadius) {
			// Wheel rotations Wr = L/C, where:
			// L - is wheel path length: L =  2 * Pi * turnRadius * angle/360
			// C - is wheel circle length: C = 2 * Pi * wheelRadius
			// Don't forget that whillRadius is mm scale, so we need devide it by 10.
			// So ttoal equation: Wr = (2 * Pi * turnRadius * angle/360) / (2 * Pi * wheelRadius/10)
			// But if we will reduce 2*Pi from equations 2 and 3, then result
			// equation will be much simpler: Wr = (turnRadius * angle/360) / (wheelRadius/10) => (turnRadius * angle/36) / wheelRadius

			/*
			 * Slow variant: return (float)(turnRadius * angle/36) / wheelRadius;
			 */

			// Fast variant: take 54bytes less and 28% less time
			return (float)(turnRadius * angle) / ((uint16_t)wheelRadius*36);
		}

		/**
		 * Calculate length of wheel path
		 * @param uint8_t turnRadius - cm
		 * @param uint16_t angle - deg
		 */
		static float turningAngleToPathDistance(uint8_t turnRadius, uint16_t angle) {
			/*
			 * Slow variant: return (float)(TWO_PI * turnRadius * angle/360);
			 */

			// Fast variant: take 150bytes less and 30% less time
			return (float)(MotionMath::F_100_TWO_PI * (uint32_t)angle * turnRadius)/MotionMath::F_100_DEG_360;
		}

	private:
};


#endif /* _GEEGROW_TRUCK_MOTION_MATH_H */
