#include "PidRegulator.h"

PidRegulator::PidRegulator() {}

PidRegulator::PidRegulator(float Kp, float Ki, float Kd) {
	this->setPidFactors(Kp, Ki, Kd);
}

void PidRegulator::setPidFactors(float Kp, float Ki, float Kd) {
	this->Kp = Kp;
	this->Ki = Ki;
	this->Kd = Kd;
}

void PidRegulator::reset() {
	this->firstRun = true;
	this->previousDifference = 0;
	this->I = 0;
}

void PidRegulator::setDebugMode(char mode) {
	debugMode = mode;
}

void PidRegulator::setResultLimits(int16_t min, int16_t max) {
	this->minLimit = min;
	this->maxLimit = max;
	this->useLimits = true;
}

void PidRegulator::resetLimits() {
	this->minLimit = 0;
	this->maxLimit = 0;
	this->useLimits = false;
}

int16_t PidRegulator::run(int16_t left, int16_t right) {
	float P, D;

	debugPrint(F("PID [ "));
	
	if (this->firstRun) {
		this->firstRun = false;
		this->previousDifference = 0;
		this->I = 0;
	}

	int16_t diff = (int16_t) (left - right);

	int16_t correction = 0;

	debugPrint(F("E=")); debugPrint(diff);

	// Compute I
	if (this->Ki) {
		correction = this->I + Ki * diff;
		this->I = correction;
		debugPrint(F(",\tI=")); debugPrint(this->I);
	}

	// Compute P
	if (this->Kp) {
		P = this->Kp * diff;
		correction += P;
		debugPrint(F(",\tP=")); debugPrint(P);
	}

	// Compute D
	if (this->Kd) {
		D = this->Kd * (diff - previousDifference);
		correction += D;
		debugPrint(F(",\tD=")); debugPrint(D);
		previousDifference = diff;
	}

	// Check correction limit
	if (this->useLimits) {
		correction = constrain(correction, this->minLimit, this->maxLimit);
	}

	debugPrint(F(",\tC=")); debugPrintln(correction);
	debugPlot(diff, P, this->I, D);

	return correction;
}

void PidRegulator::debugPlot(int16_t E, float P, float I, float D) {
	if (debugMode == 2) {
		Serial.println(E);
	} else if (debugMode == 3) {
		Serial.println(P);
	} else if (debugMode == 4) {
		Serial.println(I);
	} else if (debugMode == 5) {
		Serial.println(D);
	} else if (debugMode == 6) {
		Serial.print(E);
		Serial.print(" ");
		Serial.print(P);
		Serial.print(" ");
		Serial.print(I);
		Serial.print(" ");
		Serial.println(D);
	}
}