#ifndef _PCA9685_H
#define _PCA9685_H

#include <Arduino.h>
#include <I2CTransport.h>

#define PCA9685_FREQUENCY	50		// Default frequency of PCA9685 PWM
#define PCA9685_ADDRESS		0x40	// Address of PCA9685

#define PCA9685_SUBADR1 0x2
#define PCA9685_SUBADR2 0x3
#define PCA9685_SUBADR3 0x4

#define PCA9685_MODE1 0x0
#define PCA9685_PRESCALE 0xFE

#define LED0_ON_L 0x6
#define LED0_ON_H 0x7
#define LED0_OFF_L 0x8
#define LED0_OFF_H 0x9

#define ALLLED_ON_L 0xFA
#define ALLLED_ON_H 0xFB
#define ALLLED_OFF_L 0xFC
#define ALLLED_OFF_H 0xFD

/*
 * Because PCA9685 can't produce lower frequency..
 * ...and more than 1526 Hz also
 */
#define MINIMUM_ALLOWED_FREQUENCY 24 // Hz
#define MAXIMUM_ALLOWED_FREQUENCY 1526 // Hz


class PCA9685 {
    public:
        static PCA9685& me() {
            static PCA9685 instance;
            return instance;
        }

        static void begin(void);
        static void setPWMFreq(uint16_t frequency);
        static void setPWM(uint8_t num, uint16_t on, uint16_t off);
        static void setPWM(uint8_t num, uint8_t pwmValue);
        static void setPin(uint8_t num, uint16_t val, bool invert=false);

    private:
        PCA9685() {}
        ~PCA9685() {}
        PCA9685(const PCA9685&) = delete;
        PCA9685& operator = (PCA9685&) = delete;

        // Physical I2C address of PCA9685
        uint8_t deviceAddr = PCA9685_ADDRESS;
        uint16_t frequency = PCA9685_FREQUENCY;

        void reset(void);
        uint8_t read(uint8_t registerAddress);
        void write(uint8_t registerAddress, uint8_t data);
};

#endif
