#include <DualWheelTruck.h>

// Указатели для объектов
DCMotor* motor;
BaseTickEncoder* encoder;

// Функция для тактирования мотора
void motorTick() {
  // "Передаем" тик мотору
  motor->externalTick(1);
}

void setup() {
  Serial.begin(9600);
  delay(2000);
  /*
   * Объявляем плату. 
   * Варианты:
   *   vertor_PCA()
   *   vertor_STM()
   *   vertor_MEGA()
  */
  Board::vertor_MEGA();

  /*
   * Создаем объект мотора
   * Аргументы:
   *  [ID мотора. На Vertor Standart считам моторы слева направо, начиная с 0. Т.е. (PM0, PM1) - мотор 0, (PM2, PM3) - мотор 1 и т.д. На Vertor Mega - как подписано на плате.]
   *  [Передаточное число редуктора. Если энкодер будет установлен на выходном валу, то 1.]
   *  [Полярность подключения мотора]
   *  [Инертность колеса, от нее зависит стоп-импульс. Подбирается экспериментально.]
  */
  motor = new DCMotor(1, 1, true, 30);

  /*
   * Для подсчета количества оборотов нужно подключить энкодер, оптический или Холла.
  */

  /*
   * Создание оптического энкодера.
   * Аргументы:
   *   [Канал]
   *   [Количество секторов на одном обороте колеса энкодера]
  */
  encoder = new OpticalTickEncoder(10, 36);

  /*
   * Создание энкодера Холла.
   * Аргументы:
   *   [Канал 1]
   *   [Канал 2]
   *   [Количество импульсов на одном канале за один оборот вала двигателя]
  */
//  encoder = new HallTickEncoder(10, 8, 1);

  /*
   * Подключаем энкодер к мотору
  */
  motor->attachEncoder(encoder);

  /*
   * Для использования энкодеров и работы hard-stop необходимо тактировать мотор.
   * Для этого используем библиотеку Tweak, которая будет вызывать указанную функцию
   * через заданные интервалы времени.
   * Аргументы:
   *   [имя функции, которую надо вызывать]
   *   [интервал, мс]
  */
  Tweak::attachMsCallback(motorTick, 1);

  /*
   * Остановка мотора.
   * Аргументы:
   *   [Использовать ли hard-stop]
  */
  motor->stop(true);
}

void loop() {
  // Если мотор еще не завершил предыдущее движение, то вернуться попозже
  if (motor->isBusy()) {
    delay(100);
    return;
  }
  
  /*
   * Если подключен энкодер, то можно задавать требуемое число оборотов.
   * Аргументы:
   *   [Скорость, %]
   *   [Количество оборотов]
  */
  motor->forward(60);

  delay(5000);
  motor->backward(60);
  delay(5000);
  //Tweak::attachMsCallback(motorTick, 1);
   motor->stop(true);
    delay(5000);
  
}
