#include <SmoothServo.h>

SmoothServo* servo;

void setup() {
  /* Аргументы для конструктора SmoothServo:
    pin - номер пина в соответствии с маркировкой на плате
    mode = BOARD - способ управления сервой. По умолчанию используется BOARD, для 
                   этого нужно вызвать соответствующую функцию инициализации 
                   (см. пример). Для BOARD будут использованы соответствующие 
                   библиотеки PCA9685 и STM8_I2C. Для использования стандартной
                   библиотеки Servo от Arduino нужно указать ARDUINO_SERVO
                   вместо BOARD, но для Verter'ов это неприменимо. Есть еще две
                   опции - PCA9685 и STM8_I2C, но они сделаны под резерв для плат, 
                   которые не включены в класс Board.
    min = SERVO_DEFAULT_MIN,
    max = SERVO_DEFAULT_MAX - минимальное и максимальное значение в микросекундах
                              для значений угла 0 и 180 градусов соответственно.
                              По умолчанию 544 мкс и 2400 мкс. Эти аргументы имеют
                              смысл только при использовании платы с PCA9685. Если
                              плата с STM8, то используются фиксированные значения 
                              60 мкс и 2400 мкс, аргументы ни на что не повлияют.
    minSpeed = 30,
    maxSpeed = 100 - минимальная и максимальная угловые скорости
    acceleration = 200 - максимальное угловое ускорение
  */
  
  /* Инициализация с использованием класса Board. Включена по умолчанию */
  servo = new SmoothServo(0, BOARD, 500, 2400,30,100); // можно только с одним аргументом SmoothServo(pin);
  
  /* Инициализация используемой платы */
 // Board::vertor_PCA();  // для платы VertorStandart с PCA9685
  Board::vertor_STM();  // для платы VertorStandart с STM8
//  Board::vertor_MEGA(); // для платы VertorMega

  /* Отдельные функции для настройки скорости и ускорения, описанных в конструкторе */
  servo->setMinAngularSpeed(70);
  servo->setMaxAngularSpeed(180);
//  servo->setMaxAcceleration(200);

  /* Запуск сервы. Аргументы:
      ms = 0 - интервал дискретизации для изменения угла. Чем меньше, тем плавнее поворот.
               На практике достаточно значений 30-50.
      startAngle = 0  - начальный угол.
  */
  servo->begin(30);
}

void loop() {
  /*  Плавный поворот сервы */
  servo->smoothRotate(180);  // В аргументе передаем абсолютное значение угла, не дельту
  /* Ждем, когда серва завершит плавный поворот */
  while (servo->isBusy());  // Применятется только для ожидания завершения плавного поворота
  /*  Обычный поворот сервы */
  delay(1500);
  servo->rotate(0);  // В аргументе передаем абсолютное значение угла, не дельту
  /* Даем серве время повернуться */
  delay(2500);   // Для ожидания завершения обычного поворота задаем задержку вручную
}
