#include <DualWheelTruck.h>

/* 
 *  Создаем основной объект.
 *  Аргументы:
 *    [радиус колеса, мм]
 *    [ширина колесной базы, мм]
*/
DualWheelTruck truck = DualWheelTruck(35, 235);

// Переменная для чередования режима поворота и прямолинейного движения
uint8_t task = 0;

void setup() {
  /*
   * Объявляем, какая плата используется.
  */
  Board::vertor_MEGA();

  /*
   * Инициализация главного объекта
  */
  truck.init();

  /*
   * Создаем левый и правый моторы.
   * Аргументы:
   *  [ID мотора (как подписано на плате)]
   *  [Передаточное число редуктора. Если энкодер будет установлен на выходном валу, то 1]
   *  [Полярность подключения мотора]
   *  [Инертность колеса, от нее зависит стоп-импульс. Подбирается экспериментально.]
  */
  truck.attMotorL(2, 75, true, 40);
  truck.attMotorR(1, 75, false, 40);

  /*
   * Создаем левый и правый энкодеры Холла.
   * Аргументы:
   *  [1 канал]
   *  [2 канал]
   *  [Количество импульсов на одном канале за один оборот вала двигателя]
  */
  truck.attHallEncR(3, A3, 1);
  truck.attHallEncL(10, 19, 1);
  
  /*
   * Стоп моторы.
   * Аргументы:
   *  [использовать ли hard-stop]
  */
  truck.motorLStop(true);
  truck.motorRStop(true);

  /*
   * Задаем параметры ПИД-регулирования для прямолинейного движения и поворота.
   * Аргументы:
   *  [Пропорциональная составляющая]
   *  [Интегральная составляющая]
   *  [Дифференциальная составляющая]
  */
  truck.setPidAlign(30, 0.1, 10);
  truck.setPidDiff(10, 0.1, 5);

  /*
   * Включаем энкодеры
  */
  truck.enEncoders();
}

void loop() {
  // Проверяем, выполнено предыдущее задание или нет. Если нет, вернемся попозже.
  if (truck.isBusy()) {
    delay(100);
    return;
  }

  if (task == 0) {
    /*
     * Команда на движение вперед.
     * Аргументы:
     *  [скрорость, %]
     *  [дистанция, см]
    */
    truck.moveFwd(50, 100);
    task++;
  } else if (task == 1) {
    /*
     * Команда на поворот налево
     * Аргументы:
     *  [скрорость, %]
     *  [радиус поворота, см]
     *  [угол поворота, град]
    */
    truck.turnL(50, 50, 180);
    task = 0;
  }

  delay(1000);
}

  /********************************************************************/
  /*********************** Возможные команды **************************/
  /********************************************************************/
  /*    Управление отдельно левым мотором
    truck.motorLFwd(uint8_t speed, uint16_t distance = 0);
    truck.motorLBwd(uint8_t speed, uint16_t distance = 0);
    truck.motorLStop(bool hard = true);

        Управление отдельно правым мотором
    truck.motorRFwd(uint8_t speed, uint16_t distance = 0);
    truck.motorRBwd(uint8_t speed, uint16_t distance = 0);
    truck.motorRStop(bool hard = true);

        Управление платформой целиком
    truck.moveFwd(uint8_t speed, uint16_t distance = 0);
    truck.moveBwd(uint8_t speed, uint16_t distance = 0);
    truck.turnL(uint8_t speed, uint16_t radius, uint16_t angle);
    truck.turnR(uint8_t speed, uint16_t radius, uint16_t angle);

        Подключение оптических энкодеров
    truck.attOptEncL(uint8_t ch, uint8_t sectorsPerLoop);
    truck.attOptEncR(uint8_t ch, uint8_t sectorsPerLoop);

        Подключение энкодеров Холла
    truck.attHallEncL(uint8_t ch1, uint8_t ch2, uint8_t pulsesPerLoop);
    truck.attHallEncR(uint8_t ch1, uint8_t ch2, uint8_t pulsesPerLoop);

        Включение и выключение энкодеров
    truck.enEncoders();
    truck.disEncoders();

        Узнать пройденный путь (мм) и сбросить счетчик пути
    uint32_t truck.getDistance();
    truck.resetDistance();
  */
